//********************************************************************
//*** POSIX/sys/timeval.cc
//*** Copyright (c) 2002 by Markus Winand <mws@fatalmind.com>
//*** $Id: timeval.cc,v 1.6 2003/05/29 18:51:27 mws Exp $
//********************************************************************

#include "../../POSIX/sys/timeval.hh"

#include <iomanip>
#include <iostream>

namespace POSIX {
timeval::timeval()
{
	tv_sec = 0;
	tv_usec = 0;
}

timeval::~timeval()
{
}

timeval& timeval::operator+=(const timeval& rh)
{
	tv_sec += rh.tv_sec;
	tv_usec += rh.tv_usec;
	if (tv_usec >= 1000000) {
		tv_sec += tv_usec / 1000000;
		tv_usec %= 1000000;
	}
	return *this;
}

timeval& timeval::operator-=(const timeval& rh)
{
	tv_sec -= rh.tv_sec;
	if (tv_usec < rh.tv_usec) {
		tv_sec--;
		tv_usec += 1000000;
	}
	tv_usec -= rh.tv_usec;
	return *this;
}

timeval::operator float() const
{
	float rv(tv_usec);

	rv /= 1000000;
	rv += tv_sec;
	return rv;
}

std::ostream& operator<<(std::ostream& os, const timeval& tv)
{
	return os << tv.tv_sec << '.' << std::setw(6) << std::setfill('0') << tv.tv_usec;
}

timeval operator-(const timeval& lho, const timeval& rho)
{
	timeval rv(lho);
	rv-= rho;
	return rv;
}

}
