//********************************************************************
//*** ResourcePool/SQL/Resource.hh
//*** Copyright (c) 2003-2005 by Markus Winand <mws@fatalmind.com>
//*** $Id: Resource.hh,v 1.3 2005/03/05 01:17:39 mws Exp $
//********************************************************************
/*
This file is part of ResourcePool.

ResourcePool is free software; you can redistribute it
and/or modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2 of
the License, or (at your option) any later version.

ResourcePool is distributed in the hope that it will be
useful, but WITHOUT ANY WARRANTY; without even the implied warranty
of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with ResourcePool; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
02111-1307  USA */

#ifndef RESOURCEPOOL_SQL_RESOURCE_HH
#define RESOURCEPOOL_SQL_RESOURCE_HH

#include "mm/gc_ptr.hh"

#include "ResourcePool/Resource.hh"
#include "ResourcePool/Command.hh"
#include "ResourcePool/SQL/ResourcePoolFwd.hh"

#include <string>

namespace fatalmind {
namespace SQL {

template<class TM>
class SQLResource
: public fatalmind::Resource
{
	public:
		SQLResource();
		virtual ~SQLResource();

		virtual bool precheck() = 0;
		virtual bool postcheck() = 0;
		virtual void close() = 0;
		virtual void fail_close() = 0;

		SQLResource<TM>& get_plain_resource() {
			return *this;
		};
};

template<class TM, class RT>
class SQLResourceSpecialized
: public SQLResource<TM>
{
	public:
        SQLResourceSpecialized(RT* a_resource);
        virtual ~SQLResourceSpecialized();

		virtual bool precheck();
		virtual bool postcheck();
		virtual void close();
		virtual void fail_close();
		
		RT& getRealResource() const {
			return *resource_;
		};		
	private:
		gc_ptr<RT> resource_;
};

} // namespace SQL
} // namespace fatalmind

#endif
