//********************************************************************
//*** ResourcePool/oracle/OracleExceptionFactory.cc
//*** Copyright (c) 2003-2005 by Markus Winand <mws@fatalmind.com>
//*** $Id: OracleExceptionFactory.cc,v 1.6 2005/01/06 16:28:00 mws Exp $
//********************************************************************
/*
This file is part of ResourcePool.

ResourcePool is free software; you can redistribute it
and/or modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2 of 
the License, or (at your option) any later version.

ResourcePool is distributed in the hope that it will be
useful, but WITHOUT ANY WARRANTY; without even the implied warranty
of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with ResourcePool; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
02111-1307  USA */

#ifndef RESOURCEPOOL_ORACLE_ORACLEEXCEPTIONFACTORY_HH
#include "OracleExceptionFactory.hh"
#endif

#ifndef RESOURCEPOOL_ORACLE_RESOURCE_HH
#include "Resource.hh"
#endif

namespace fatalmind {

void OracleExceptionFactory::throwit(const std::string& op
	, const std::string& fac
	, const int errorcode
	, const std::string& msg
	, const std::string& sqlstm
) {
	std::ostringstream os;
	os << "ResourcePool/Error(" << op << "): " 
		<< fac << "-" << errorcode << ": " << msg;
	throw NFOOracleException(os.str(), errorcode, sqlstm);
}

void OracleExceptionFactory::throwit(const std::string& op
	, const oracle::OCIEnvHandle& env
	, sword ret
	, const std::string& sqlstm
) {
	return throwit(op, env.get(), ret, OCI_HTYPE_ENV, sqlstm);
}

void OracleExceptionFactory::throwit(const std::string& op
	, const oracle::OCIErrorHandle& err
	, sword ret
	, const std::string& sqlstm
) {
	return throwit(op, err.get(), ret, OCI_HTYPE_ERROR, sqlstm);
}

void OracleExceptionFactory::throwit(const std::string& op
	, const oracleHandles& hand
	, sword ret
	, const std::string& sqlstm
) {
	return throwit(op, hand.getError(), ret, OCI_HTYPE_ERROR, sqlstm);
}

void OracleExceptionFactory::throwit(const std::string& oper
	, const dvoid* errhp
	, sword ret
	, ub4 type
	, const std::string& sqlstm
) {
	std::string msg;
	int errorcode;
	switch (ret) {
		case OCI_SUCCESS_WITH_INFO:
			msg = "OCI_SUCCESS_WITH_INFO";
			break;
		case OCI_NEED_DATA:
			msg = "OCI_NEED_DATA";
			break;
		case OCI_NO_DATA:
			msg = "OCI_NO_DATA";
			errorcode = 999999;
			break;
		case OCI_ERROR: {
			OraText msgbuf[1024];

			OCIErrorGet((dvoid*)errhp
				, (ub4) 1
				, (OraText *) NULL
				, &errorcode
				, msgbuf
				, (ub4) sizeof(msgbuf)
				, (ub4) type
			);
			std::ostringstream os;
			os << "OCI_ERROR(" << oper << "): " << msgbuf;
			msg = os.str();
			}
			break;
		case OCI_INVALID_HANDLE:
			{
			std::ostringstream os;
			os << oper << ": " << "OCI_INVALID_HANDLE";
			msg = os.str();
			}
			break;
		case OCI_STILL_EXECUTING:
			msg = "OCI_STILL_EXECUTING";
			break;
		case OCI_CONTINUE:
			msg = "OCI_CONTINUE";
			break;
	};
	std::string state(OracleException::mapORAtoXOPEN(errorcode));
	if (!state.compare(0,2, "08")) {
		throw OracleException(msg, errorcode, sqlstm);
	} else {
		throw NFOOracleException(msg, errorcode, sqlstm);
	}
}

} // namespace fatalmind

