//********************************************************************
//*** ResourcePool/oracle/Transaction.cc
//*** Copyright (c) 2003-2005 by Markus Winand <mws@fatalmind.com>
//*** $Id: Transaction.cc,v 1.17 2005/01/16 10:39:04 mws Exp $
//********************************************************************
/*
This file is part of ResourcePool.

ResourcePool is free software; you can redistribute it
and/or modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2 of 
the License, or (at your option) any later version.

ResourcePool is distributed in the hope that it will be
useful, but WITHOUT ANY WARRANTY; without even the implied warranty
of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with ResourcePool; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
02111-1307  USA */

#ifndef RESOURCEPOOL_ORACLE_TRANSACTION_HH
#include "Transaction.hh"
#endif

#ifndef RESOURCEPOOL_ORACLE_RESOURCE_HH
#include "Resource.hh"
#endif

#ifndef RESOURCEPOOL_ORACLE_ORACLEEXCEPTIONFACTORY_HH
#include "OracleExceptionFactory.hh"
#endif

#ifndef RESOURCEPOOL_ORACLE_RESOURCEPOOL_HH
#include "ResourcePool.hh"
#endif

namespace fatalmind {
namespace oracle {

template<class TM, template<class> class SLMDC>
Transaction<TM, SLMDC>::Transaction()
: _list()
//, _sthcache()
{
	_list.reserve(20);
}

template<class TM, template<class> class SLMDC>
Transaction<TM, SLMDC>::~Transaction() {
}

template<class TM, template<class> class SLMDC>
CommandInterface<TM, SLMDC>* Transaction<TM, SLMDC>::clone() const {
	return new Transaction(*this);
}

template<class TM, template<class> class SLMDC>
void Transaction<TM, SLMDC>::autoCommit(bool ac) {
	if (!_list.empty()) {
		_list.back()->autoCommit(ac);
	}
}

template<class TM, template<class> class SLMDC>
void Transaction<TM, SLMDC>::add(const CommandInterface<TM, SLMDC>& cmd) 
{
	_list.push_back(gc_ptr<CommandInterface<TM, SLMDC> >(cmd.clone()));
	_list.back()->autoCommit(false);
}

template<class TM, template<class> class SLMDC>
void Transaction<TM, SLMDC>::commit(oracleHandlesWithCache<TM>& hand) {
	sword ret;
	if ((ret = OCITransCommit(hand.getSvcCtx()
		, hand.getError()
		, OCI_DEFAULT
	)) != OCI_SUCCESS) {
		OracleExceptionFactory::throwit("OCITransCommit", hand, ret, "commit");
	}
}

template<class TM, template<class> class SLMDC>
void Transaction<TM, SLMDC>::rollback(oracleHandlesWithCache<TM>& hand) {
	sword ret;
	if ((ret = OCITransRollback(hand.getSvcCtx()
		, hand.getError()
		, OCI_DEFAULT
	)) != OCI_SUCCESS) {
		OracleExceptionFactory::throwit("OCITransRollback", hand, ret, "rollback");
	}
}

template<class TM, template<class> class SLMDC>
void Transaction<TM, SLMDC>::execute(oracleHandlesWithCache<TM>& hand) {
	try {
		if (!_list.empty()) {
			_list.back()->autoCommit();
			StatementCache sthcache;
			doit(hand, sthcache);
		}
		//commit(hand);
	} catch (...) {
		rollback(hand);
		throw;
	}
}

template<class TM, template<class> class SLMDC>
void Transaction<TM, SLMDC>::_execute(oracleHandlesWithCache<TM>& hand, StatementCache& sthcache) {
	if (!_list.empty()) {
		doit(hand, sthcache);
	}
}

template<class TM, template<class> class SLMDC>
void Transaction<TM, SLMDC>::doit(oracleHandlesWithCache<TM>& hand, StatementCache& sthcache) {
	typename _list_t::iterator end = _list.end();
	for (typename _list_t::iterator i = _list.begin(); i != end; ++i) {
		(*i)->_execute(hand, sthcache);
	}
}

template class Transaction<DefaultThreadedModel, oracle::SelectListMetaDataCache >;
template class Transaction<DefaultThreadedModel, oracle::SelectListMetaDataNopCache >;

} // namespace oracle
} // namespace fatalmind
