//********************************************************************
//*** Thread/TaskRunner.cc
//*** Copyright (c) 2003-2005 by Markus Winand <mws@fatalmind.com>
//*** $Id: TaskRunner.cc,v 1.9 2005/01/06 16:28:06 mws Exp $
//********************************************************************
/*
This file is part of ResourcePool.

ResourcePool is free software; you can redistribute it
and/or modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2 of 
the License, or (at your option) any later version.

ResourcePool is distributed in the hope that it will be
useful, but WITHOUT ANY WARRANTY; without even the implied warranty
of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with ResourcePool; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
02111-1307  USA */

#ifndef INCLUDED_TASKRUNNER_HH
#include "TaskRunner.hh"
#endif

using namespace std;

namespace fatalmind {

TaskRunner::Stopped::Stopped(string msg)
: Exception(msg)
{
}

TaskRunner::TaskRunner()
: _condvar()
, _tsk()
, _stopped(false)
, _wo()
{
	start();
}

TaskRunner::~TaskRunner()
{
	try {
		stop();
	} catch (...) {
	}
}

void TaskRunner::waitForTsk() {	
	while (_tsk == 0 && !_stopped) {
		_condvar.wait();
	}
}

void TaskRunner::waitForSlot() {	
	while (_tsk != 0 && !_stopped) {
		_condvar.wait();
	}
	if (_stopped) {
		throw Stopped("The TaskRunner has already been stopped");
	}
}

void TaskRunner::run() {
	sync_t lk(_condvar);
	waitForTsk();
	while (! _stopped) {
		try {
			_tsk->run();
		} catch (...) {
			// TODO:
		}
		if (_wo.get() != 0) {
			_wo->done();
		}
		_wo.reset();
		_tsk = 0;
		_condvar.signal();
		waitForTsk();
	};
}

void TaskRunner::runDetachedLocked(Task& tsk) 
{
	_tsk = &dynamic_cast<Task&>(tsk);
	_condvar.broadcast();
}

void TaskRunner::runDetached(Task& tsk) 
{
	sync_t lk(_condvar);
	waitForSlot();
	_wo.reset();
	runDetachedLocked(tsk);
}

TaskRunner::ID TaskRunner::run(Task& tsk) {
	sync_t lk(_condvar);
	waitForSlot();
	gc_ptr<WaitObject, MultiThreadedModel<> > wc(new WaitObject());
	_wo = wc;
	TaskRunner::ID id(_wo);
	runDetachedLocked(tsk);
	return id;
}

void TaskRunner::stop() {
	try {
		sync_t lk(_condvar);
		waitForSlot();
		_stopped = true;
		_condvar.broadcast();
	} catch (...) {
	}
	join();
}

/*
 *
 * WaitObject
 *
 */

TaskRunner::WaitObject::WaitObject()
: _condvar()
, _done(false)
{
}

TaskRunner::WaitObject::~WaitObject()
{
}

void TaskRunner::WaitObject::done()
{
	sync_t lk(_condvar);
	_done = true;
	_condvar.broadcast();
}

void TaskRunner::WaitObject::wait() 
{
	sync_t lk(_condvar);
	while (! _done) {
		_condvar.wait();
	}
}

/*
 *
 * ID
 *
 */

TaskRunner::ID::ID() 
: _wo()
{
}

TaskRunner::ID::ID(gc_ptr<WaitObject, MultiThreadedModel<> >& wo) 
: _wo(wo)
{
}

TaskRunner::ID& TaskRunner::ID::operator=(const ID& rho)
{
	_wo = rho._wo;
	return *this;
}

void TaskRunner::ID::wait()
{
	_wo->wait();
}

void TaskRunner::ID::detach()
{
	_wo.reset();
}

} // namespace fatalmind
