//********************************************************************
//*** Thread/TaskRunner.hh
//*** Copyright (c) 2003-2005 by Markus Winand <mws@fatalmind.com>
//*** $Id: TaskRunner.hh,v 1.10 2005/01/06 16:28:06 mws Exp $
//********************************************************************
/*
This file is part of ResourcePool.

ResourcePool is free software; you can redistribute it
and/or modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2 of 
the License, or (at your option) any later version.

ResourcePool is distributed in the hope that it will be
useful, but WITHOUT ANY WARRANTY; without even the implied warranty
of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with ResourcePool; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
02111-1307  USA */

#ifndef INCLUDED_TASKRUNNER_HH
#define INCLUDED_TASKRUNNER_HH

#ifndef INCLUDED_THREAD_H
#include "Thread.hh"
#endif

#ifndef INCLUDED_THREAD_LOCK_HH
#include "Lock.hh"
#endif

#ifndef INCLUDED_THREAD_SYNCHRONIZE_HH
#include "Thread/Synchronize.hh"
#endif

#ifndef INCLUDED_EXCEPTIONS_HH
#include "Exceptions/Exception.hh"
#endif

#ifndef INCLUDED_GC_PTR_HH
#include "mm/gc_ptr.hh"
#endif

namespace fatalmind {

class TaskRunner: public Thread {
	protected:
		class WaitObject {
			public:
				WaitObject();
				~WaitObject();
				
				void done();
				void wait();
			private:
				typedef FastLock<>  lock_t;
				typedef Synchronize<lock_t> sync_t;
				CondVar<lock_t>     _condvar;
				bool                _done;

				WaitObject(const WaitObject&); // c-ctor
		};
	public:
		TaskRunner();
		virtual ~TaskRunner();

		class ID {
			public:
				ID();
				ID(const ID&);	
				ID(gc_ptr<WaitObject, MultiThreadedModel<> >&);
				ID& operator=(const ID&);
				void wait();
				void detach();
			private:
				gc_ptr<WaitObject, MultiThreadedModel<> > _wo;
		};

		virtual void runDetached(Task&);
		virtual ID run(Task&);
		virtual void stop();

		class Stopped: public Exception {
			public:
				Stopped(string msg);
		};

	protected:
		virtual void run();
		
	private:
		void waitForTsk();
		void waitForSlot();
		void runDetachedLocked(Task&);

		TaskRunner(TaskRunner&); // c-ctor is hidden

		typedef FastLock<>  lock_t;
		typedef Synchronize<lock_t> sync_t;

		CondVar<lock_t>   _condvar;	
		Task*             _tsk;
		bool              _stopped;
		gc_ptr<WaitObject, MultiThreadedModel<> > _wo;
};

} //namespace fatalmind

#endif
