//********************************************************************
//*** cache/Cache.cc
//*** Copyright (c) 2003-2005 by Markus Winand <mws@fatalmind.com>
//*** $Id: Cache.cc,v 1.7 2005/01/06 16:28:08 mws Exp $
//********************************************************************

#ifndef CACHE_CACHE_HH
#include "Cache.hh"
#endif

#ifndef INCLUDED_EXCEPTIONS_HH
#include "Exceptions/Exception.hh"
#endif

#ifndef INCLUDED_THREAD_SYNCHRONIZE_HH
#include "Thread/Synchronize.hh"
#endif

#include "ExtremeFinder.cc"

//#include <algorithm>

namespace fatalmind {

template<typename K, typename V, typename Ctor, typename E, class TM, template<typename> class Allocator>
Cache<K, V, Ctor, E, TM, Allocator>::Cache(
	  const unsigned int maxsize
	, const unsigned int expirecnt
):_maxsize(maxsize)
, _expirecnt(expirecnt?expirecnt: int(maxsize * 0.07)?int(maxsize * 0.07):1)
{
}

template<typename K, typename V, typename Ctor, typename E, class TM, template<typename> class Allocator>
typename Cache<K, V, Ctor, E, TM, Allocator>::super::value_t 
Cache<K, V, Ctor, E, TM, Allocator>::get(typename arg_traits<K>::const_param key)
{
	Synchronize<_lock_t> sync(_lk);
	typename _map_t::const_iterator end = _map.end();
	typename _map_t::iterator hit = _map.find(key);
	if (hit != end) {
		if (!expire_invalid(hit)) {
			hit->second.first.hit();
			return hit->second.second;
		}
	} 
	return typename super::value_t();
}

template<typename K, typename V, typename Ctor, typename E, class TM, template<typename> class Allocator>
typename Cache<K, V, Ctor, E, TM, Allocator>::super::value_t 
Cache<K, V, Ctor, E, TM, Allocator>::get(
	  typename arg_traits<K>::const_param key
	, typename arg_traits<Ctor>::const_param arg
) {
	typename super::value_t ret = get(key);
	if (ret.isNull()) {
		ret = put(key, typename super::value_t(arg()));
	}
	return ret;
}
template<typename K, typename V, typename Ctor, typename E, class TM, template<typename> class Allocator>
typename Cache<K, V, Ctor, E, TM, Allocator>::super::value_t 
Cache<K, V, Ctor, E, TM, Allocator>::put(
	  typename arg_traits<K>::const_param key
	, typename super::value_t val
) {
	Synchronize<_lock_t> sync(_lk);
	_make_space_for_one();	
	std::pair<typename _map_t::iterator, bool> rv =
		_map.insert(
			std::make_pair(key, std::make_pair(E(), val))
		);
	if (rv.second) {
		return (rv.first)->second.second;
	} else {
		throw Exception("impossible ;), maybe out of memory??");
	}	
}

/*
 *
 * internal stuff
 *
 */
template<typename K, typename V, typename Ctor, typename E, class TM, template<typename> class Allocator>
void Cache<K, V, Ctor, E, TM, Allocator>::_make_space_for_one()
{
	if (full()) {
		ExtremeFinder<int, typename _map_t::iterator> _ef(_expirecnt);	
		bool rv = false;
		typename _map_t::const_iterator end = _map.end();
		for (typename _map_t::iterator i = _map.begin(); i != end && !rv; ++i) {
			rv = rv || expire_invalid(i);
			_ef.put(i->second.first.efficiency(), i);
		}
		if (! rv) {
			typename ExtremeFinder<int, typename _map_t::iterator>::iterator end = _ef.end();
			for (typename ExtremeFinder<int, typename _map_t::iterator>::iterator i = _ef.begin(); i != end; ++i) {
				expire(i->second);
			}
		}
	}
}

template<typename K, typename V, typename Ctor, typename E, class TM, template<typename> class Allocator>
void Cache<K, V, Ctor, E, TM, Allocator>::expire(typename _map_t::iterator hit)
{
	_map.erase(hit);
}

template<typename K, typename V, typename Ctor, typename E, class TM, template<typename> class Allocator>
bool Cache<K, V, Ctor, E, TM, Allocator>::expire_invalid()
{
	bool rv = false;
	typename _map_t::const_iterator end = _map.end();
	for (typename _map_t::iterator i = _map.begin(); i != end; ++i) {
		rv = rv || expire_invalid(i);	
	}
	return rv;
}

template<typename K, typename V, typename Ctor, typename E, class TM, template<typename> class Allocator>
bool Cache<K, V, Ctor, E, TM, Allocator>::expire_invalid(typename _map_t::iterator hit)
{
	if (! hit->second.first.isValid()) {
		expire(hit);
		return true;;
	}
	return false;
}


template<typename K, typename V, typename Ctor, typename E, class TM, template<typename> class Allocator>
bool Cache<K, V, Ctor, E, TM, Allocator>::full() const
{
	return _maxsize <= _map.size();
}

}// namespace fatalmind
